@@ -40,7 +40,7 @@
 #include "timehead.h"
 
 #define DRIVER_NAME	"Microsol Solis UPS driver"
-#define DRIVER_VERSION	"0.61"
+#define DRIVER_VERSION	"0.62"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -610,7 +610,7 @@ static void ScanReceivePack( void )
 
 	/* Autonomy */
 
-	if( ( Autonomy < 5 ) )
+	if( Autonomy < 5 )
 		LowBatt = true;
 	else
 		LowBatt = false;
@@ -618,7 +618,7 @@ static void ScanReceivePack( void )
 	UpsPowerFactor = 700;
 
 	/* input 110V or 220v */
-	if(  ( InputValue == 0 ) )  {
+	if(  InputValue == 0 )  {
 		InDownLim = 75;
 		InUpLim = 150;
 		NomInVolt = 110;
@@ -702,7 +702,7 @@ CommReceive(const char *bufptr,  int size)
 
 	int i, CheckSum, i_end;
 
-	if(  ( size==25 ) )
+	if(  size==25 )
 		Waiting = 0;
 	
 	switch( Waiting )
@@ -727,7 +727,8 @@ CommReceive(const char *bufptr,  int size)
 		ser_flush_in(upsfd,"",0); /* clean port */
   
 		/* correct package */
-		if(  ( (RecPack[0] & 0xF0) == 0xA0 )
+		/* 0xA0 is original solis.c; 0xB0 is for APC-branded Microsol units */
+		if(  ( ( (RecPack[0] & 0xF0) == 0xA0 ) || (RecPack[0] & 0xF0) == 0xB0)
 		       && ( RecPack[ 24 ] == 254 )
 		       && ( RecPack[ 23 ] == CheckSum ) ) {
 
@@ -742,7 +743,11 @@ CommReceive(const char *bufptr,  int size)
 
 			switch( SolisModel )
 			{
-			case 10:
+			case 10: /* Added for APC-Branded Microsol units */
+				{
+				ScanReceivePack();	
+				break;	
+				}
 			case 11:
 			case 12:
 			case 13:
@@ -755,6 +760,7 @@ CommReceive(const char *bufptr,  int size)
 			default:
 			{
 				printf( M_UNKN );
+				ScanReceivePack(); // Scan anyway.
 				break;
 			}
 		}
@@ -884,26 +890,30 @@ static void getbaseinfo(void)
 
 	switch( SolisModel )
 	{
-	case 10:
+	case 10: /* Added for APC-Microsol units */
+	{
+		Model = "Back-UPS 1200 BR";
+		break;
+	}
 	case 11:
 	case 12:
 	{
-		strcpy(Model, "Solis 1.0");
+		Model = "Solis 1.0";
 		break;
 	}
 	case 13:
 	{
-		strcpy(Model, "Solis 1.5");
+		Model = "Solis 1.5";
 		break;
 	}
 	case 14:
 	{
-		strcpy(Model, "Solis 2.0");
+		Model = "Solis 2.0";
 		break;
 	}
 	case 15:
 	{
-		strcpy(Model, "Solis 3.0");
+		Model = "Solis 3.0";
 		break;
 	}
 	}
