@@ -52,6 +52,7 @@ static usb_device_id_t usb_device_table[] = {
 	{ 0x047c, 0xffff, "usbhid-ups" },
 	{ 0x04b4, 0x5500, "riello_usb" },
 	{ 0x04d8, 0xd004, "usbhid-ups" },
+	{ 0x04d8, 0xd005, "usbhid-ups" },
 	{ 0x050d, 0x0375, "usbhid-ups" },
 	{ 0x050d, 0x0551, "usbhid-ups" },
 	{ 0x050d, 0x0750, "usbhid-ups" },
@@ -60,14 +61,16 @@ static usb_device_id_t usb_device_table[] = {
 	{ 0x050d, 0x0910, "usbhid-ups" },
 	{ 0x050d, 0x0912, "usbhid-ups" },
 	{ 0x050d, 0x0980, "usbhid-ups" },
+	{ 0x050d, 0x0f51, "usbhid-ups" },
 	{ 0x050d, 0x1100, "usbhid-ups" },
+	{ 0x051d, 0x0000, "usbhid-ups" },
 	{ 0x051d, 0x0002, "usbhid-ups" },
 	{ 0x051d, 0x0003, "usbhid-ups" },
 	{ 0x0592, 0x0002, "bcmxcp_usb" },
 	{ 0x0592, 0x0004, "usbhid-ups" },
 	{ 0x05b8, 0x0000, "blazer_usb" },
 	{ 0x0665, 0x5161, "blazer_usb" },
-	{ 0x06da, 0x0002, "blazer_usb" },
+	{ 0x06da, 0x0002, "bcmxcp_usb" },
 	{ 0x06da, 0x0003, "blazer_usb" },
 	{ 0x06da, 0x0004, "blazer_usb" },
 	{ 0x06da, 0x0005, "blazer_usb" },
@@ -111,6 +114,7 @@ static usb_device_id_t usb_device_table[] = {
 	{ 0x09ae, 0x4006, "usbhid-ups" },
 	{ 0x09ae, 0x4007, "usbhid-ups" },
 	{ 0x09ae, 0x4008, "usbhid-ups" },
+	{ 0x0d9f, 0x0001, "usbhid-ups" },
 	{ 0x0d9f, 0x0004, "usbhid-ups" },
 	{ 0x0d9f, 0x00a2, "usbhid-ups" },
 	{ 0x0d9f, 0x00a3, "usbhid-ups" },
@@ -120,6 +124,7 @@ static usb_device_id_t usb_device_table[] = {
 	{ 0x0f03, 0x0001, "blazer_usb" },
 	{ 0x10af, 0x0001, "usbhid-ups" },
 	{ 0x10af, 0x0004, "usbhid-ups" },
+	{ 0x10af, 0x0008, "usbhid-ups" },
 	{ 0x14f0, 0x00c9, "blazer_usb" },
 	{ 0xffff, 0x0000, "blazer_usb" },
 	/* Terminating entry */
